@@ -17,31 +17,42 @@ from __future__ import unicode_literals
 django_doc_major_version = '1.6'
 
 #: The version range required for Django.
-django_version = '>=1.6.11,<1.8.999'
+django_version = '>=1.6.11,<1.10.999'
 
 #: Dependencies required for LessCSS pipelining.
 lesscss_npm_dependencies = {
-    'less': '2.7.1',
+    'less': '^3.11.0',
+    'less-plugin-autoprefix': '^2.0.0',
 }
 
 #: Dependencies required for UglifyJS JavaScript compression.
 uglifyjs_npm_dependencies = {
-    'uglifyjs': '2.4.10',
+    'uglify-js': '^2.8.29',
 }
 
-#: All static media dependencies required to package/develop against Djblets.
+#: Dependencies required for Babel for JavaScript.
+babel_npm_dependencies = {
+    'babel-cli': '^6.26.0',
+    'babel-preset-es2015': '^6.24.1',
+    'babel-plugin-dedent': '^2.0.0',
+    'babel-plugin-django-gettext': '^1.0.0',
+}
+
+#: All static media dependencies required to package/develop against  Djblets.
 npm_dependencies = {}
 npm_dependencies.update(lesscss_npm_dependencies)
 npm_dependencies.update(uglifyjs_npm_dependencies)
+npm_dependencies.update(babel_npm_dependencies)
 
 #: All dependencies required to install Djblets.
 package_dependencies = {
     'Django': django_version,
-    'django-pipeline': '>=1.3.23,<1.3.9999',
+    'django-pipeline': '>=1.6.14,<1.6.999',
     'dnspython': '>=1.14.0',
     'feedparser': '>=5.1.2',
     'pillowfight': '',
     'publicsuffix': '>=1.1',
+    'python-dateutil': '>=1.5',
     'pytz': '',
 }
 
