@@ -1,27 +1,4 @@
-#
-# extension.py -- Base classes for extensions
-#
-# Copyright (c) 2010-2013  Beanbag, Inc.
-# Copyright (c) 2008-2010  Christian Hammond
-#
-# Permission is hereby granted, free of charge, to any person obtaining
-# a copy of this software and associated documentation files (the
-# "Software"), to deal in the Software without restriction, including
-# without limitation the rights to use, copy, modify, merge, publish,
-# distribute, sublicense, and/or sell copies of the Software, and to
-# permit persons to whom the Software is furnished to do so, subject to
-# the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+"""Base classes for implementing extensions."""
 
 from __future__ import unicode_literals
 
@@ -32,12 +9,16 @@ import os
 import warnings
 from email.parser import FeedParser
 
+import pkg_resources
 from django.conf import settings
+from django.contrib.staticfiles.templatetags.staticfiles import static
 from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import get_mod_func
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext as _
 
+from djblets.deprecation import RemovedInDjblets20Warning
+from djblets.extensions.errors import InstallExtensionMediaError
 from djblets.extensions.settings import Settings
 from djblets.util.decorators import cached_property
 
@@ -334,9 +315,26 @@ class Extension(object):
 
     def shutdown_hooks(self):
         """Shut down all hooks for the extension."""
-        for hook in self.hooks:
+        for hook in self.hooks.copy():
             if hook.initialized:
-                hook.shutdown()
+                hook.disable_hook()
+
+    def get_static_url(self, path):
+        """Return the URL to a static media file for this extension.
+
+        This takes care of resolving the static media path name to a path
+        relative to the web server. If a versioned media file is found, it
+        will be used, so that browser-side caching can be used.
+
+        Args:
+            path (unicode):
+                The path within the static directory for the extension.
+
+        Returns:
+            unicode:
+            The resulting static media URL.
+        """
+        return static('ext/%s/%s' % (self.id, path))
 
     def get_bundle_id(self, name):
         """Return the ID for a CSS or JavaScript bundle.
@@ -441,7 +439,8 @@ class ExtensionInfo(object):
                              'the extension may be missing.',
                              extension_id)
 
-        data = '\n'.join(lines)
+        # Both Python 2 and 3 will use byte strings here.
+        data = b'\n'.join(lines)
 
         # Try to decode the PKG-INFO content. If no decoding method is
         # successful then the PKG-INFO content will remain unchanged and
@@ -460,6 +459,8 @@ class ExtensionInfo(object):
         p.feed(data)
         pkg_info = p.close()
 
+        # Convert from a Message to a dictionary. Note that items() is correct
+        # here. six.iteritems() will not work.
         return dict(pkg_info.items())
 
     def __init__(self, ext_class, package_name, metadata={}):
@@ -508,7 +509,7 @@ class ExtensionInfo(object):
                     'ExtensionInfo.__init__() no longer accepts an '
                     'EntryPoint. Please update your code to call '
                     'ExtensionInfo.create_from_entrypoint() instead.',
-                    DeprecationWarning)
+                    RemovedInDjblets20Warning)
             else:
                 logger.error('Unexpected parameters passed to '
                              'ExtensionInfo.__init__: ext_class=%r, '
@@ -520,6 +521,7 @@ class ExtensionInfo(object):
 
         # Set the base information from the extension and the package.
         self.package_name = package_name
+        self.module_name = ext_class.__module__
         self.app_name = '.'.join(ext_class.__module__.split('.')[:-1])
         self.is_configurable = ext_class.is_configurable
         self.has_admin_site = ext_class.has_admin_site
@@ -548,5 +550,101 @@ class ExtensionInfo(object):
         self.url = metadata.get('Home-page')
         self.author_url = metadata.get('Author-home-page', self.url)
 
+    @cached_property
+    def installed_static_version_path(self):
+        """The path to the static media version file.
+
+        This file records the version of the extension used when last
+        installing the static media files.
+
+        Type:
+            unicode
+        """
+        return os.path.join(self.installed_static_path, '.version')
+
+    def has_resource(self, path):
+        """Return whether an extension has a resource in its package.
+
+        A resource is a file or directory that exists within an extension's
+        package.
+
+        Args:
+            path (unicode):
+                The ``/``-delimited path to the resource within the package.
+
+        Returns:
+            bool:
+            ``True`` if the resource exits. ``False`` if it does not.
+        """
+        return pkg_resources.resource_exists(self.module_name, path)
+
+    def extract_resource(self, path):
+        """Return the filesystem path to an extracted resource.
+
+        A resource is a file or directory that exists within an extension's
+        package.
+
+        This will extract the resource from the package, if the package is
+        compressed, and then return the local path to the file on the
+        filesystem.
+
+        Args:
+            path (unicode):
+                The ``/``-delimited path to the resource within the package.
+
+        Returns:
+            unicode:
+            The local filesystem path to the resource, or ``None`` if it
+            could not be found.
+        """
+        if self.has_resource(path):
+            return pkg_resources.resource_filename(self.module_name, path)
+
+        return None
+
+    def write_installed_static_version(self):
+        """Write the extension's current static media version to disk.
+
+        This will write the extension's current version in its static media
+        directory, creating that directory if necessary. This will allow
+        the extension manager to check if new media files need to be installed.
+
+        Raises:
+            djblets.extensions.errors.InstallExtensionMediaError:
+                There was an error writing the version to the static media
+                directory. Details are in the error message.
+        """
+        version_path = self.installed_static_version_path
+        parent_path = os.path.dirname(version_path)
+
+        try:
+            if not os.path.exists(parent_path):
+                os.makedirs(parent_path, 0o755)
+
+            with open(version_path, 'w') as fp:
+                fp.write('%s\n' % self.version)
+        except Exception:
+            raise InstallExtensionMediaError(
+                _('Unable to write the extension static media version '
+                  'file "%(path)s". Please make sure the file and its parent '
+                  'directory are owned by the web server.')
+                % {
+                    'path': version_path,
+                })
+
+    def get_installed_static_version(self):
+        """Return the extension's locally-written static media version.
+
+        Returns:
+            unicode:
+            The extension version written to disk, or ``None`` if it didn't
+            exist or couldn't be read.
+        """
+        try:
+            with open(self.installed_static_version_path, 'r') as fp:
+                return fp.read().strip()
+        except IOError:
+            return None
+
     def __str__(self):
-        return "%s %s (enabled = %s)" % (self.name, self.version, self.enabled)
+        return '%s %s (enabled = %s)' % (self.name, self.version, self.enabled)
