@@ -1,10 +1,11 @@
 """Packaging support for extensions."""
 
-from __future__ import unicode_literals
+from __future__ import print_function, unicode_literals
 
 import inspect
 import json
 import os
+import re
 import sys
 from distutils.errors import DistutilsExecError
 from fnmatch import fnmatch
@@ -15,7 +16,8 @@ from django.utils import six
 from setuptools.command.build_py import build_py
 from setuptools import Command
 
-from djblets.dependencies import (lesscss_npm_dependencies,
+from djblets.dependencies import (babel_npm_dependencies,
+                                  lesscss_npm_dependencies,
                                   uglifyjs_npm_dependencies)
 from djblets.util.filesystem import is_exe_in_path
 
@@ -98,13 +100,15 @@ class BuildStaticFiles(Command):
                 staticfile_dir = staticfile_dir[1]
 
             dirname = os.path.dirname(staticfile_dir)
+            less_include.add(dirname)
 
             if os.path.basename(dirname) == 'static':
-                # Static files should be living within a static/ directory,
-                # generally-speaking. That's how they'll be referenced when
-                # using @{STATIC_ROOT}. If we find a static file dir living
-                # inside a static/ directory, strip away the static/ so that
-                # lookups will work.
+                # This code is here for compatibility reasons. With a proper
+                # Babel setup, .less files should be able to reference others
+                # relative to static/ directories. However, some paths may
+                # still prefix /static/ to any paths, and we don't want those
+                # to break. So, add the parent of this static/ directory
+                # in order to allow those lookups to continue working.
                 dirname = os.path.dirname(dirname)
 
             less_include.add(dirname)
@@ -153,7 +157,14 @@ class BuildStaticFiles(Command):
             dependencies.update(uglifyjs_npm_dependencies)
 
             pipeline_settings.UGLIFYJS_BINARY = \
-                os.path.join(node_modules_dir, 'uglifyjs', 'bin', 'uglifyjs')
+                os.path.join(node_modules_dir, 'uglify-js', 'bin', 'uglifyjs')
+
+            if self.get_bundle_file_matches(js_bundles, '*.es6.js'):
+                dependencies.update(babel_npm_dependencies)
+
+                pipeline_settings.BABEL_BINARY = \
+                    os.path.join(node_modules_dir, 'babel-cli', 'bin',
+                                 'babel.js')
 
         package_file = os.path.join(build_dir, 'package.json')
 
@@ -228,10 +239,10 @@ class BuildStaticFiles(Command):
             if not os.path.exists('node_modules'):
                 os.mkdir('node_modules', 0755)
 
-            print 'Installing %s...' % package_spec
+            print('Installing %s...' % package_spec)
             result = os.system('npm install %s' % package_spec)
         else:
-            print 'Installing node packages...'
+            print('Installing node packages...')
             result = os.system('npm install')
 
         if result != 0:
@@ -298,6 +309,7 @@ class BuildStaticFiles(Command):
 
     def _build_static_media(self, extension):
         from django.conf import settings
+        from pipeline.conf import settings as pipeline_settings
 
         pipeline_js = {}
         pipeline_css = {}
@@ -325,36 +337,26 @@ class BuildStaticFiles(Command):
             PackagingFinder.extension_static_dir
         ]
 
-        # Register the include path and any global variables used for
-        # building .less files.
-        settings.PIPELINE_LESS_ARGUMENTS = ' '.join(
-            [
-                '--include-path=%s'
-                % os.path.pathsep.join(self.get_lessc_include_path())
-            ] + [
-                '--global-var="%s=%s"'
-                % (key, self._serialize_lessc_value(value))
-                for key, value in six.iteritems(self.get_lessc_global_vars())
-            ]
-        )
-
-        settings.PIPELINE_JS = pipeline_js
-        settings.PIPELINE_CSS = pipeline_css
-        settings.PIPELINE_ENABLED = True
-        settings.PIPELINE_STORAGE = \
-            'djblets.extensions.staticfiles.PackagingStorage'
         settings.STATIC_ROOT = \
             os.path.join(self.build_lib,
                          os.path.relpath(os.path.join(module_dir, 'static')))
 
+        # Tell djblets.pipeline.compiles.less.LessCompiler (if enabled) not
+        # to check for outdated using its special import script, and to in
+        # fact assume all files are outdated so they'll all be rebuilt. This
+        # will also prevent access denied issues (due to referenced files not
+        # being in the sandboxed tree defined for the static media).
+        pipeline_settings._DJBLETS_LESS_ALWAYS_REBUILD = True
+
         # Due to how Pipeline copies and stores its settings, we actually
         # have to copy over some of these, as they'll be from the original
         # loaded settings.
         from pipeline.conf import settings as pipeline_settings
 
-        for key in six.iterkeys(pipeline_settings.__dict__):
-            if hasattr(settings, key):
-                setattr(pipeline_settings, key, getattr(settings, key))
+        pipeline_settings.JAVASCRIPT = pipeline_js
+        pipeline_settings.STYLESHEETS = pipeline_css
+        pipeline_settings.PIPELINE_ENABLED = True
+        pipeline_settings.LESS_ARGUMENTS = self._build_lessc_args()
 
         # Collect and process all static media files.
         call_command('collectstatic', interactive=False, verbosity=2)
@@ -365,6 +367,49 @@ class BuildStaticFiles(Command):
             self._remove_source_files(
                 pipeline_js, os.path.join(settings.STATIC_ROOT, 'js'))
 
+    def _build_lessc_args(self):
+        """Build the list of arguments for the less compiler.
+
+        This will return a set of arguments that define any global variables
+        or include paths for the less compiler. It respects most existing
+        arguments already in ``settings.PIPELINE['LESS_ARGUMENTS']``, but
+        will filter out any existing include paths and global variables needed
+        for packaging.
+
+        Returns:
+            list:
+            A list of arguments to pass to lessc.
+        """
+        from django.conf import settings
+
+        # Get all the arguments that the application defines, but remove those
+        # that we need to override or the ones that aren't compatible with
+        # building this media.
+        lessc_global_vars = self.get_lessc_global_vars()
+        exclude_re = re.compile(
+            '^--(include-path=|global-var="?(%s)=)'
+            % '|'.join(
+                re.escape(global_var)
+                for global_var in six.iterkeys(lessc_global_vars)
+            ))
+
+        lessc_args = [
+            lessc_arg
+            for lessc_arg in settings.PIPELINE.get('LESS_ARGUMENTS', [])
+            if not exclude_re.match(lessc_arg)
+        ]
+
+        # Register the include path and any global variables used for
+        # building .less files.
+        return lessc_args + [
+            '--include-path=%s'
+            % os.path.pathsep.join(self.get_lessc_include_path())
+        ] + [
+            '--global-var=%s=%s'
+            % (key, self._serialize_lessc_value(value))
+            for key, value in six.iteritems(lessc_global_vars)
+        ]
+
     def _add_bundle(self, pipeline_bundles, extension_bundles, default_dir,
                     ext):
         for name, bundle in six.iteritems(extension_bundles):
@@ -404,7 +449,7 @@ class BuildStaticFiles(Command):
 
     def _serialize_lessc_value(self, value):
         if isinstance(value, six.text_type):
-            return '\\"%s\\"' % value
+            return '"%s"' % value
         elif isinstance(value, bool):
             if value:
                 return 'true'
