@@ -6,21 +6,28 @@ from xml.dom.minidom import parseString
 
 from django.utils import six
 from django.utils.six.moves import cStringIO as StringIO
+from django.utils.six.moves.html_entities import name2codepoint
 from markdown import Markdown, markdownFromFile
 
 
 default_app_config = 'djblets.markdown.apps.MarkdownAppConfig'
 
 
-MARKDOWN_SPECIAL_CHARS = re.escape(r''.join(Markdown.ESCAPED_CHARS))
+MARKDOWN_ALL_ESCAPED_CHARS = (
+    '\\', '`', '*', '_', '{', '}', '[', ']', '(', ')', '>', '#',
+    '+', '-', '.', '!',
+)
+
+MARKDOWN_SPECIAL_CHARS = re.escape(r''.join(MARKDOWN_ALL_ESCAPED_CHARS))
 MARKDOWN_SPECIAL_CHARS_RE = re.compile(r'([%s])' % MARKDOWN_SPECIAL_CHARS)
 
 # Markdown.ESCAPED_CHARS lists several characters to escape, but it's not
 # that simple. We only want to escape certain things if they'll actually affect
 # the Markdown rendering, because otherwise it's annoying to look at the
 # source.
-MARKDOWN_ESCAPED_CHARS = set(Markdown.ESCAPED_CHARS)
-MARKDOWN_ESCAPED_CHARS -= set(['.', '#', '-', '+', '_', '(', ')', '*', '>'])
+MARKDOWN_ESCAPED_CHARS = set(MARKDOWN_ALL_ESCAPED_CHARS) - {
+    '.', '#', '-', '+', '_', '(', ')', '*', '>'
+}
 
 ESCAPE_CHARS_RE = re.compile(r"""
     (
@@ -123,10 +130,33 @@ def iter_markdown_lines(markdown_html):
             if (node.tagName == 'div' and
                 node.attributes.get('class', 'codehilite')):
                 # This is a code block, which will consist of a bunch of lines
-                # for the source code. We want to split that up into
-                # individual lines with their own <pre> tags.
-                for line in node.toxml().splitlines():
-                    yield '<pre>%s</pre>' % line
+                # for the source code. It contains a single <pre> element
+                # within it that contains the full code block. We want to split
+                # that up into individual lines which are each wrapped in their
+                # own <div class="codehilite"><pre> elements.
+                child_lines = node.firstChild.toxml().splitlines()
+
+                # The first and last lines in child_lines begin and end with
+                # <pre> and </pre>, respectively. Clear that out.
+                assert child_lines[0].startswith('<pre>')
+                assert child_lines[-1].endswith('</pre>')
+                child_lines[0] = child_lines[0][len('<pre>'):]
+                child_lines[-1] = child_lines[-1][:-len('</pre>')]
+                if not child_lines[-1]:
+                    child_lines = child_lines[:-1]
+
+                fmt = '<div class="codehilite%s"><pre>%s</pre></div>'
+
+                # If there's just a single line, output that directly
+                if len(child_lines) == 1:
+                    yield fmt % ('', child_lines[0])
+                elif len(child_lines) > 1:
+                    yield fmt % (' codehilite-multiline-start', child_lines[0])
+
+                    for line in child_lines[1:-1]:
+                        yield fmt % (' codehilite-multiline-middle', line)
+
+                    yield fmt % (' codehilite-multiline-end', child_lines[-1])
             elif node.tagName in ('ul', 'ol'):
                 # This is a list. We'll need to split all of its items
                 # into individual lists, in order to retain bullet points
@@ -171,14 +201,37 @@ def get_markdown_element_tree(markdown_html):
     """Return an XML element tree for Markdown-generated HTML.
 
     This will build the tree and return all nodes representing the rendered
-    Markdown content.
+    Markdown content. The element tree is generated using Python's
+    :py:mod:`xml.dom.minidom`.
+
+    Args:
+        markdown_html (bytes or unicode):
+            The Markdown-generated HTML to parse.
+
+    Returns:
+        xml.dom.minicompat.NodeList:
+            The list of nodes representing the Markdown-generated HTML.
     """
     markdown_html = sanitize_illegal_chars_for_xml(markdown_html)
 
-    if isinstance(markdown_html, six.text_type):
-        markdown_html = markdown_html.encode('utf-8')
+    # Python-Markdown converts all characters in an e-mail address to
+    # entities (likely to avoid e-mail address harvesting). It optimistically
+    # tries to use named HTML entities for special characters, but these won't
+    # be known to an XML parser.
+    #
+    # Since there's no easy way to register entities with xml.dom.minidom, and
+    # we don't want to change the return type, the easiest solution is to
+    # convert named entites back to character codes through a regex before
+    # parsing.
+    unknown_code = ord('?')
+    markdown_html = re.sub(
+        r'&([^#][^;]+);',
+        lambda m: '&#%s;' % name2codepoint.get(m.group(1), unknown_code),
+        markdown_html,
+        flags=re.M)
+
+    doc = parseString(('<html>%s</html>' % markdown_html).encode('utf-8'))
 
-    doc = parseString(b'<html>%s</html>' % markdown_html)
     return doc.childNodes[0].childNodes
 
 
