@@ -26,7 +26,6 @@
 
 from __future__ import print_function, unicode_literals
 
-import copy
 import imp
 import inspect
 import os
@@ -34,6 +33,9 @@ import re
 import socket
 import sys
 import threading
+from collections import OrderedDict
+from contextlib import contextmanager
+from importlib import import_module
 
 from django.conf import settings
 from django.core import serializers
@@ -43,21 +45,22 @@ from django.core.management import call_command
 from django.core.servers import basehttp
 from django.db import (DatabaseError, DEFAULT_DB_ALIAS, IntegrityError,
                        connections, router)
-from django.db.models import Model, get_apps
-from django.db.models.loading import cache, load_app
+from django.db.models import Model
 from django.template import Node
 from django.test import testcases
 from django.utils import six
-from django.utils.importlib import import_module
 from django.utils.module_loading import module_has_submodule
 
 try:
+    # Django >= 1.7
     from django.apps import apps
-    from django.db import migrations
 except ImportError:
     # Django < 1.7
+    from django.db.models.loading import cache as app_cache, load_app
+
     apps = None
-    migrations = None
+
+from djblets.siteconfig.models import SiteConfiguration
 
 
 class StubNodeList(Node):
@@ -89,18 +92,18 @@ class TestCase(testcases.TestCase):
 
     def __call__(self, *args, **kwargs):
         method = getattr(self, self._testMethodName)
-        old_fixtures = getattr(self, 'fixtures', [])
+        old_fixtures = getattr(self, 'fixtures', None)
 
         if hasattr(method, '_fixtures'):
-            if getattr(method, '_replace_fixtures'):
+            if (getattr(method, '_replace_fixtures') or
+                old_fixtures is None):
                 self.fixtures = method._fixtures
             else:
                 self.fixtures = old_fixtures + method._fixtures
 
         super(TestCase, self).__call__(*args, **kwargs)
 
-        if old_fixtures:
-            self.fixtures = old_fixtures
+        self.fixtures = old_fixtures
 
     def shortDescription(self):
         """Returns the description of the current test.
@@ -117,6 +120,40 @@ class TestCase(testcases.TestCase):
 
         return doc
 
+    @contextmanager
+    def siteconfig_settings(self, settings):
+        """Temporarily sets siteconfig settings for a test.
+
+        Subclasses should override this if they want to run a method like
+        :py:func:`~djblets.siteconfig.django_settings.apply_django_settings`
+        before and after each test.
+
+        Args:
+            settings (dict):
+                The new siteconfig settings to set.
+
+        Context:
+            The current site configuration will contain the new settings for
+            this test.
+        """
+        siteconfig = SiteConfiguration.objects.get_current()
+
+        old_settings = {}
+
+        for key, value in six.iteritems(settings):
+            old_settings[key] = siteconfig.get(key)
+            siteconfig.set(key, value)
+
+        siteconfig.save()
+
+        try:
+            yield
+        finally:
+            for key, value in six.iteritems(old_settings):
+                siteconfig.set(key, value)
+
+            siteconfig.save()
+
     def assertRaisesValidationError(self, expected_messages, *args, **kwargs):
         """Assert that a ValidationError is raised with the given message(s).
 
@@ -194,13 +231,15 @@ class TestModelsLoaderMixin(object):
         if not cls.tests_app:
             cls.tests_app = cls.__module__
 
-        tests_module = import_module(cls.tests_app)
+        models_mod_name = '%s.models' % cls.tests_app
 
-        if not module_has_submodule(tests_module, 'models'):
+        try:
+            models_mod = import_module(models_mod_name)
+        except ImportError:
             # Set up a 'models' module, containing any models local to the
             # module that this TestCase is in.
-            models_mod_name = '%s.models' % cls.tests_app
             models_mod = imp.new_module(models_mod_name)
+            module_name = cls.__module__
 
             # Django needs a value here. Doesn't matter what it is.
             models_mod.__file__ = ''
@@ -209,10 +248,12 @@ class TestModelsLoaderMixin(object):
             test_module = sys.modules[cls.__module__]
 
             for key, value in six.iteritems(test_module.__dict__):
-                if inspect.isclass(value) and issubclass(value, Model):
+                if (inspect.isclass(value) and
+                    issubclass(value, Model) and
+                    value.__module__ == module_name):
                     models_mod.__dict__[key] = value
 
-            cls._tests_loader_models_mod = models_mod
+        cls._tests_loader_models_mod = models_mod
 
     @classmethod
     def tearDownClass(cls):
@@ -241,11 +282,25 @@ class TestModelsLoaderMixin(object):
             apps.set_installed_apps(settings.INSTALLED_APPS)
             app_config = apps.get_containing_app_config(self.tests_app)
 
-            for key, value in six.iteritems(models_mod.__dict__):
-                if inspect.isclass(value) and issubclass(value, Model):
-                    apps.register_model(app_config.label, value)
+            if models_mod:
+                app_label = app_config.label
 
-            call_command('migrate', verbosity=0, interactive=False)
+                for key, value in six.iteritems(models_mod.__dict__):
+                    if inspect.isclass(value) and issubclass(value, Model):
+                        # The model was likely registered under another app,
+                        # so we need to remove the old one and add the new
+                        # one.
+                        try:
+                            del apps.all_models[value._meta.app_label][
+                                value._meta.model_name]
+                        except KeyError:
+                            pass
+
+                        value._meta.app_label = app_label
+                        apps.register_model(app_label, value)
+
+            call_command('migrate', run_syncdb=True, verbosity=0,
+                         interactive=False)
         else:
             load_app(self.tests_app)
             call_command('syncdb', verbosity=0, interactive=False)
@@ -268,12 +323,12 @@ class TestModelsLoaderMixin(object):
 
         if apps:
             apps.unset_installed_apps()
-            cache.clear_cache()
+            apps.all_models[self.tests_app].clear()
         else:
             if models_mod:
-                del cache.app_store[models_mod]
+                del app_cache.app_store[models_mod]
 
-            cache._get_models_cache.clear()
+            app_cache._get_models_cache.clear()
 
 
 class FixturesCompilerMixin(object):
@@ -385,7 +440,7 @@ class FixturesCompilerMixin(object):
         if not self._fixture_dirs:
             app_module_paths = []
 
-            for app in get_apps():
+            for app in self._get_app_model_modules():
                 if hasattr(app, '__path__'):
                     # It's a 'models/' subpackage.
                     for path in app.__path__:
@@ -420,34 +475,109 @@ class FixturesCompilerMixin(object):
             db (unicode):
                 The database name to load fixture data on.
         """
-        models = set()
+        table_names = set()
         connection = connections[db]
 
         with connection.constraint_checks_disabled():
             for fixture in fixtures:
                 assert db in self._precompiled_fixtures
                 assert fixture in self._precompiled_fixtures[db]
-                objects = self._precompiled_fixtures[db][fixture]
+                deserialized_objs = self._precompiled_fixtures[db][fixture]
 
-                for obj in objects:
-                    models.add(obj.object.__class__)
+                to_save = OrderedDict()
+                to_save_m2m = []
 
+                # Start off by going through the list of deserialized object
+                # information from the fixtures and group them into categories
+                # that we'll further filter down later.
+                for deserialized_obj in deserialized_objs:
+                    obj = deserialized_obj.object
+
+                    table_names.add(obj._meta.db_table)
+                    to_save.setdefault(type(obj), []).append(obj)
+
+                    if deserialized_obj.m2m_data:
+                        to_save_m2m.append((obj, deserialized_obj.m2m_data))
+
+                # Now we'll break this down into objects we can batch-create
+                # and ones we have to update.
+                #
+                # The database may already have entries for some of these
+                # models, particularly if they're being shared across multiple
+                # tests for the same suite. In that case, when doing a save()
+                # on an object, Django will perform a update-or-create
+                # operation, involving an UPDATE statement followed by an
+                # INSERT If the UPDATE returned 0 rows.
+                #
+                # That's wasteful. Instead, we're going to just delete
+                # everything matching the deserialized objects' IDs and then
+                # bulk-create new entries.
+                try:
+                    for model_cls, objs in six.iteritems(to_save):
+                        obj_ids = [
+                            _obj.pk
+                            for _obj in objs
+                            if _obj.pk
+                        ]
+
+                        if obj_ids:
+                            model_cls.objects.filter(pk__in=obj_ids).delete()
+
+                        model_cls.objects.using(db).bulk_create(objs)
+                except (DatabaseError, IntegrityError) as e:
+                    meta = model_cls._meta
+                    sys.stderr.write(
+                        'Could not load %s.%s from fixture "%s": %s\n'
+                        % (meta.app_label,
+                           meta.object_name,
+                           fixture,
+                           e))
+                    raise
+
+                # Now save any Many-to-Many field relations.
+                #
+                # Note that we're not assigning the values to the
+                # ManyToManyField attribute. Instead, we're getting the add()
+                # method and calling that. We can trust that the relations are
+                # empty (since we've newly-created the object), so we don't
+                # need to clear the old list first.
+                for obj, m2m_data in to_save_m2m:
                     try:
-                        obj = copy.copy(obj)
-                        obj.save(using=db)
+                        for m2m_attr, m2m_objs in six.iteritems(m2m_data):
+                            getattr(obj, m2m_attr).add(*m2m_objs)
                     except (DatabaseError, IntegrityError) as e:
-                        sys.stderr.write('Could not load %s.%s(pk=%s): %s\n'
-                                         % (obj.object._meta.app_label,
-                                            obj.object._meta.object_name,
-                                            obj.object.pk,
-                                            e))
+                        meta = obj._meta
+                        sys.stderr.write(
+                            'Could not load Many-to-Many entries for %s.%s '
+                            '(pk=%s) in fixture "%s": %s\n'
+                            % (meta.app_label,
+                               meta.object_name,
+                               obj.pk,
+                               fixture,
+                               e))
                         raise
 
         # We disabled constraints above, so check now.
-        connection.check_constraints(table_names=[
-            model._meta.db_table
-            for model in models
-        ])
+        connection.check_constraints(table_names=sorted(table_names))
+
+    def _get_app_model_modules(self):
+        """Return the entire list of registered Django app models modules.
+
+        This is an internal utility function that's used to provide
+        compatibility with all supported versions of Django.
+
+        Returns:
+            list:
+            The list of Django applications.
+        """
+        if apps:
+            return [
+                app.models_module
+                for app in apps.get_app_configs()
+                if app.models_module
+            ]
+        else:
+            return app_cache.get_apps()
 
     def __getattribute__(self, name):
         if name == 'fixtures' and self.__dict__.get('_hide_fixtures'):
