@@ -27,16 +27,19 @@ from __future__ import unicode_literals
 
 import datetime
 import os
+import re
 
 from django import template
-from django.template import TemplateSyntaxError
+from django.template import TemplateSyntaxError, Variable
 from django.template.defaultfilters import stringfilter
-from django.template.loader import render_to_string
+from django.utils import six
+from django.utils.html import escape, format_html, strip_spaces_between_tags
 from django.utils.safestring import mark_safe
 from django.utils.six.moves.urllib.parse import urlencode
 from django.utils.timezone import is_aware
 
-from djblets.util.decorators import basictag, blocktag
+from djblets.util.compat.django.template.loader import render_to_string
+from djblets.util.decorators import blocktag
 from djblets.util.dates import get_tz_aware_utcnow
 from djblets.util.http import get_url_params_except
 from djblets.util.humanize import humanize_list
@@ -45,32 +48,76 @@ from djblets.util.humanize import humanize_list
 register = template.Library()
 
 
+WS_RE = re.compile('\s+')
+
+
 @register.tag
-@blocktag
-def definevar(context, nodelist, varname):
+@blocktag(resolve_vars=False)
+def definevar(context, nodelist, varname, *options):
     """Define a variable for later use in the template.
 
     The variable defined can be used within the same context (such as the
     same block or for loop). This is useful for caching a portion of a
     template that would otherwise be expensive to repeatedly compute.
 
+    .. versionadded:: 1.0
+
+       Added new ``strip``, ``spaceless``, and ``unsafe`` options.
+
     Args:
         varname (unicode):
             The variable name.
 
+        *options (list of unicode, optional):
+            A list of options passed. This supports the following:
+
+            ``strip``:
+                Strip whitespace at the beginning/end of the value.
+
+            ``spaceless``:
+                Strip whitespace at the beginning/end and remove all spaces
+                between tags. This implies ``strip``.
+
+            ``unsafe``:
+                Mark the text as unsafe. The contents will be HTML-escaped when
+                inserted into the page.
+
         block_content (unicode):
             The block content to set in the variable.
 
     Example:
         .. code-block:: html+django
 
-           {% definevar "myvar" %}{% expensive_tag %}{% enddefinevar %}
+           {% definevar "myvar1" %}
+           {%  expensive_tag %}
+           {% enddefinevar %}
 
-           {{myvar}}
-           {{myvar}}
+           {% definevar "myvar2" spaceless %}
+           <div>
+            <a href="#">Click me!</a>
+           </div>
+           {% enddefinevar %}
+
+           {{myvar1}}
+           {{myvar2}}
     """
-    context[varname] = nodelist.render(context)
-    return ""
+    varname = Variable(varname).resolve(context)
+    result = nodelist.render(context)
+
+    if 'spaceless' in options:
+        result = strip_spaces_between_tags(result.strip())
+    elif 'strip' in options:
+        result = result.strip()
+
+    if 'unsafe' in options:
+        # Unicode strings are inherently "unsafe".
+        result = six.text_type(result)
+    else:
+        result = mark_safe(result)
+
+    context[varname] = result
+
+    return ''
 
 
 @register.tag
@@ -168,8 +215,7 @@ def _check_userorperm(context, user, perm):
             user == req_user.pk)
 
 
-@register.tag
-@basictag(takes_context=True)
+@register.simple_tag(takes_context=True)
 def include_as_string(context, template_name):
     """Include the contents of a template as an escaped string.
 
@@ -194,21 +240,46 @@ def include_as_string(context, template_name):
     s = render_to_string(template_name, context)
     s = s.replace("'", "\\'")
     s = s.replace("\n", "\\\n")
-    return "'%s'" % s
+
+    # Since this works like {% include %}, we have to trust the resulting
+    # content here. It's still possible that a nefarious template could cause
+    # problems, but this is the responsibility of the caller.
+    #
+    # In prior versions of Django (< 1.9), this was implicitly marked safe.
+    return mark_safe("'%s'" % s)
 
 
 @register.tag
-@blocktag
-def attr(context, nodelist, attrname):
+@blocktag(resolve_vars=False)
+def attr(context, nodelist, attrname, *options):
     """Set an HTML attribute to a value if the value is not an empty string.
 
     This is a handy way of adding attributes with non-empty values to an
     HTML element without requiring several `{% if %}` tags.
 
+    The contents will be stripped and all whitespace within condensed before
+    being considered or rendered. This can be turned off (restoring pre-1.0
+    behavior) by passing ``nocondense`` as an option.
+
+    .. versionchanged:: 1.0
+
+       Prior to this release, all whitespace before/after/within the
+       attribute value was preserved. Now ``nocondense`` is required for this
+       behavior.
+
+       The value is now escaped as well. Previously the value was assumed to
+       be safe, requiring the consumer to escape the contents.
+
     Args:
         attrname (unicode):
             The name for the HTML attribute.
 
+        *options (list unicode, optional):
+            A list of options passed. This supports the following:
+
+            ``nocondense``:
+                Preserves all whitespace in the value.
+
         block_content (unicode):
             The block content to render for the attribute value.
 
@@ -219,14 +290,23 @@ def attr(context, nodelist, attrname):
     Example:
         .. code-block:: html+django
 
-           <div{% attr "data-description" %}{{obj.description}}{% endattr %}>
+           <div{% attr "class" %}{{obj.name}}{% endattr %}>
+           <div{% attr "data-description" nocondense %}
+               Space-sensitive
+
+               whitespace
+           {% endattr %}>
     """
+    attrname = Variable(attrname).resolve(context)
     content = nodelist.render(context)
 
-    if content.strip() == "":
-        return ""
+    if 'nocondense' not in options:
+        content = WS_RE.sub(' ', content.strip())
 
-    return ' %s="%s"' % (attrname, content)
+    if not content:
+        return ''
+
+    return format_html(' {0}="{1}"', attrname, six.text_type(content))
 
 
 @register.filter
@@ -646,8 +726,7 @@ def split(s, delim=','):
     return s.split(delim)
 
 
-@register.tag
-@basictag(takes_context=True)
+@register.simple_tag(takes_context=True)
 def querystring_with(context, attr, value):
     """Return the current page URL with a new query string argument added.
 
@@ -681,6 +760,8 @@ def querystring_with(context, attr, value):
     new_query = urlencode({attr.encode('utf-8'): value.encode('utf-8')})
 
     if existing_query:
-        return '?%s&%s' % (existing_query, new_query)
+        result = '?%s&%s' % (existing_query, new_query)
     else:
-        return '?%s' % new_query
+        result = '?%s' % new_query
+
+    return escape(result)
