@@ -5,20 +5,20 @@ from __future__ import unicode_literals
 import logging
 import warnings
 
-from django.conf.urls import include, patterns, url
+from django.conf.urls import include, url
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse
 from django.db import models
-from django.db.models.fields.related import (
-    ManyRelatedObjectsDescriptor,
-    ReverseManyRelatedObjectsDescriptor,
-    ReverseSingleRelatedObjectDescriptor)
 from django.db.models.query import QuerySet
 from django.http import (HttpResponseNotAllowed, HttpResponse,
                          HttpResponseNotModified)
 from django.utils import six
 from django.views.decorators.vary import vary_on_headers
 
+from djblets.auth.ratelimit import (RATE_LIMIT_API_ANONYMOUS,
+                                    RATE_LIMIT_API_AUTHENTICATED,
+                                    get_usage_count)
+from djblets.deprecation import RemovedInDjblets20Warning
 from djblets.util.http import (get_modified_since, encode_etag,
                                etag_if_none_match,
                                set_last_modified, set_etag,
@@ -39,8 +39,30 @@ from djblets.webapi.errors import (DOES_NOT_EXIST,
                                    LOGIN_FAILED,
                                    NOT_LOGGED_IN,
                                    PERMISSION_DENIED,
+                                   RATE_LIMIT_EXCEEDED,
                                    WebAPIError)
 
+try:
+    # Django >= 1.9
+    from django.db.models.fields.related_descriptors import (
+        ManyToManyDescriptor,
+        ForwardManyToOneDescriptor)
+
+    m2m_descriptors = (ManyToManyDescriptor,)
+    fkey_descriptors = (ForwardManyToOneDescriptor,)
+except ImportError:
+    # Django < 1.9
+    from django.db.models.fields.related import (
+        ManyRelatedObjectsDescriptor,
+        ReverseManyRelatedObjectsDescriptor,
+        ReverseSingleRelatedObjectDescriptor)
+
+    m2m_descriptors = (
+        ManyRelatedObjectsDescriptor,
+        ReverseManyRelatedObjectsDescriptor,
+    )
+    fkey_descriptors = (ReverseSingleRelatedObjectDescriptor,)
+
 
 logger = logging.getLogger(__name__)
 
@@ -153,6 +175,29 @@ class WebAPIResource(object):
                     api_format=api_format,
                     mimetype=self._build_error_mimetype(request))
 
+        # Check to see if the user has been rate limited. Rate limits for
+        # authentication failures have already been handled via the auth
+        # backend, so this just checks for the API rate limits, if any.
+        if auth_result:
+            rate_limit_type = RATE_LIMIT_API_AUTHENTICATED
+        else:
+            rate_limit_type = RATE_LIMIT_API_ANONYMOUS
+
+        usage = get_usage_count(request, increment=True,
+                                limit_type=rate_limit_type)
+
+        if (usage is not None and
+            usage['count'] > usage['limit']):
+            return WebAPIResponseError(
+                request,
+                err=RATE_LIMIT_EXCEEDED,
+                headers={
+                    'Retry-After': usage['time_left'],
+                    'X-RateLimit-Limit': usage['limit'],
+                },
+                api_format=api_format,
+                mimetype=self._build_error_mimetype(request))
+
         method = request.method
 
         if method == 'POST':
@@ -258,17 +303,6 @@ class WebAPIResource(object):
         else:
             return HttpResponseNotAllowed(self.allowed_methods)
 
-    def call_method_view(self, request, method, view, *args, **kwargs):
-        """Calls the given method view.
-
-        This will just call the given view by default, passing in all
-        args and kwargs.
-
-        This can be overridden by subclasses to perform additional
-        checks or pass additional data to the view.
-        """
-        return view(request, *args, **kwargs)
-
     @property
     def __name__(self):
         return self.__class__.__name__
@@ -323,32 +357,16 @@ class WebAPIResource(object):
         """
         return self.name_plural
 
-    def _build_resource_mimetype(self, mimetype, is_list):
-        if is_list:
-            resource_name = (self.mimetype_list_resource_name or
-                             self.name_plural.replace('_', '-'))
-        else:
-            resource_name = (self.mimetype_item_resource_name or
-                             self.name.replace('_', '-'))
+    def call_method_view(self, request, method, view, *args, **kwargs):
+        """Calls the given method view.
 
-        return self._build_vendor_mimetype(mimetype, resource_name)
+        This will just call the given view by default, passing in all
+        args and kwargs.
 
-    def _build_error_mimetype(self, request):
-        mimetype = get_http_requested_mimetype(
-            request, WebAPIResponse.supported_mimetypes)
-
-        if self.mimetype_vendor:
-            mimetype = self._build_vendor_mimetype(mimetype, 'error')
-
-        return mimetype
-
-    def _build_vendor_mimetype(self, mimetype, name):
-        parts = mimetype.split('/')
-
-        return '%s/vnd.%s.%s+%s' % (parts[0],
-                                    self.mimetype_vendor,
-                                    name,
-                                    parts[1])
+        This can be overridden by subclasses to perform additional
+        checks or pass additional data to the view.
+        """
+        return view(request, *args, **kwargs)
 
     def build_response_args(self, request):
         is_list = (request._djblets_webapi_method == 'GET' and
@@ -645,10 +663,9 @@ class WebAPIResource(object):
         for resource in self.list_child_resources:
             resource._parent_resource = self
             child_regex = r'^' + resource.uri_name + r'/'
-            urlpatterns += patterns(
-                '',
+            urlpatterns += [
                 url(child_regex, include(resource.get_url_patterns())),
-            )
+            ]
 
         if self.uri_object_key or self.singleton:
             # If the resource has particular items in it...
@@ -667,10 +684,9 @@ class WebAPIResource(object):
             for resource in self.item_child_resources:
                 resource._parent_resource = self
                 child_regex = base_regex + resource.uri_name + r'/'
-                urlpatterns += patterns(
-                    '',
+                urlpatterns += [
                     url(child_regex, include(resource.get_url_patterns())),
-                )
+                ]
 
         return urlpatterns
 
@@ -722,9 +738,33 @@ class WebAPIResource(object):
         return {
             'method': 'GET',
             'href': resource.get_href(obj, *args, **kwargs),
-            'title': six.text_type(obj),
+            'title': resource.get_object_title(obj, *args, **kwargs),
         }
 
+    def get_object_title(self, obj, request=None, *args, **kwargs):
+        """Return the object's title.
+
+        By default, this returns the object's unicode representation.
+
+        Args:
+            obj (object):
+                The object to serialize.
+
+            request (django.http.HttpRequest):
+                The current request.
+
+            *args (tuple):
+                Additional positional arguments.
+
+            **kwargs (dict):
+                Additional keyword arguments.
+
+        Returns:
+            unicode:
+            The object's title.
+        """
+        return six.text_type(obj)
+
     def serialize_object(self, obj, *args, **kwargs):
         """Serializes the object into a Python dictionary."""
         request = kwargs.get('request', None)
@@ -898,19 +938,6 @@ class WebAPIResource(object):
         """
         return self._get_only_items(request, 'only-links', 'only_links')
 
-    def _get_only_items(self, request, query_param_name, post_field_name):
-        if request:
-            only = request.GET.get(query_param_name,
-                                   request.POST.get(post_field_name, None))
-
-            if only is not None:
-                if only:
-                    return only.split(',')
-                else:
-                    return []
-
-        return None
-
     def get_serializer_for_object(self, obj):
         """Returns the serializer used to serialize an object.
 
@@ -1172,10 +1199,10 @@ class WebAPIResource(object):
         # Future versions of Djblets will remove the encode_etag argument.
         if encode_etag:
             warnings.warn('WebAPIResource.generate_etag will stop generating '
-                          'encoded ETags in 0.9.x. Update your get_etag() '
+                          'encoded ETags in 2.0.x. Update your get_etag() '
                           'method to pass encode_etag=False to this function '
                           'and to call encode_etag() on the result instead.',
-                          DeprecationWarning)
+                          RemovedInDjblets20Warning)
             etag = self.encode_etag(request, etag)
 
         return etag
@@ -1211,10 +1238,50 @@ class WebAPIResource(object):
         else:
             return NOT_LOGGED_IN
 
+    def _build_resource_mimetype(self, mimetype, is_list):
+        if is_list:
+            resource_name = (self.mimetype_list_resource_name or
+                             self.name_plural.replace('_', '-'))
+        else:
+            resource_name = (self.mimetype_item_resource_name or
+                             self.name.replace('_', '-'))
+
+        return self._build_vendor_mimetype(mimetype, resource_name)
+
+    def _build_error_mimetype(self, request):
+        mimetype = get_http_requested_mimetype(
+            request, WebAPIResponse.supported_mimetypes)
+
+        if self.mimetype_vendor:
+            mimetype = self._build_vendor_mimetype(mimetype, 'error')
+
+        return mimetype
+
+    def _build_vendor_mimetype(self, mimetype, name):
+        parts = mimetype.split('/')
+
+        return '%s/vnd.%s.%s+%s' % (parts[0],
+                                    self.mimetype_vendor,
+                                    name,
+                                    parts[1])
+
     def _build_named_url(self, name):
         """Builds a Django URL name from the provided name."""
         return '%s-resource' % name.replace('_', '-')
 
+    def _get_only_items(self, request, query_param_name, post_field_name):
+        if request:
+            only = request.GET.get(query_param_name,
+                                   request.POST.get(post_field_name, None))
+
+            if only is not None:
+                if only:
+                    return only.split(',')
+                else:
+                    return []
+
+        return None
+
     def _get_queryset(self, request, is_list=False, *args, **kwargs):
         """Returns an optimized queryset.
 
@@ -1233,9 +1300,7 @@ class WebAPIResource(object):
 
                 field_type = getattr(self.model, field, None)
 
-                if (field_type and
-                    isinstance(field_type,
-                               ReverseSingleRelatedObjectDescriptor)):
+                if field_type and isinstance(field_type, fkey_descriptors):
                     self._select_related_fields.append(field)
 
         if self._select_related_fields:
@@ -1252,10 +1317,7 @@ class WebAPIResource(object):
 
                     field_type = getattr(self.model, field, None)
 
-                    if (field_type and
-                        isinstance(field_type,
-                                   (ReverseManyRelatedObjectsDescriptor,
-                                    ManyRelatedObjectsDescriptor))):
+                    if field_type and isinstance(field_type, m2m_descriptors):
                         self._prefetch_related_fields.append(field)
 
             if self._prefetch_related_fields:
